import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.imageio.*;
import javax.swing.*;
 
public class WaterMark extends JFrame
{
	/*
	 * Widgets.
	 */
	JTextField	SourceFileBox			=	new JTextField( 15 );
	JTextField	WatermarkFileBox		=	new JTextField( 15 );
	JFileChooser	SourceFileChooser		=	new JFileChooser();
	JFileChooser	WatermarkFileChooser		=	new JFileChooser();	
	JButton		ButtonCreate			=	new JButton( "Create" );
	JButton		SourceButton			=	new JButton( "Select Source Image" );
	JButton		WaterMarkButton			=	new JButton( "Select Watermark Image" );
	
	String[]	SupportedFormats		=	{ "png", "jpeg", "gif", "bmp" };
	String ChosenFormat				=	"png";
	JComboBox	FormatChooser			=	new JComboBox( SupportedFormats );
	
	String[]	WatermarkPositions		=	{ "centre", "bottom right" };
	int		ChosenPosition			=	1;
	JComboBox	PositionChooser			=	new JComboBox( WatermarkPositions );
	
	JLabel		LabelFormat			=	new JLabel( "Output Format:" );
	JLabel		LabelPosition			=	new JLabel( "Watermark Position:" );
	
	
	/*
	 * Constructor.
	 * Build the awful swing UI.
	 */
	WaterMark()
	{
		// Disable Filename Boxes.
		SourceFileBox.setEditable( false );
		WatermarkFileBox.setEditable( false );
		
		/*
		 * Assign Actions.
		 */
		SourceButton.addActionListener( new SourceButtonAction() );
		WaterMarkButton.addActionListener( new WaterMarkButtonAction() );
		ButtonCreate.addActionListener( new ButtonCreateAction() );
		FormatChooser.addActionListener( new FormatChooserAction() );
		PositionChooser.addActionListener( new PositionChooserAction() );
		
		/*
 		 * Create Content Pane.
 		 */
 		JPanel ContentArea			=	new JPanel( ); 
		ContentArea.setLayout( new GridLayout(13, 0) );
		ContentArea.add( SourceFileBox );
		ContentArea.add( SourceButton );
		ContentArea.add( Box.createVerticalGlue() );

		ContentArea.add( WatermarkFileBox );
		ContentArea.add( WaterMarkButton );
		ContentArea.add( Box.createVerticalGlue() );

		ContentArea.add( LabelFormat );
		ContentArea.add( FormatChooser );
		ContentArea.add( Box.createVerticalGlue() );
		
		ContentArea.add( LabelPosition );
		ContentArea.add( PositionChooser );
		ContentArea.add( Box.createVerticalGlue() );

		ContentArea.add( ButtonCreate );

		ContentArea.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		

    
		this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		this.setContentPane( ContentArea );
		this.pack();
		this.setTitle( "Watermark" );
		this.setResizable( false );
		this.setVisible( true );
	}
	
	
	/*
	 * Main function.
	 * Only here to call MakeWatermark() for portability reasons.
	 */
	public static void main( String[] args ) throws IOException
	{
	        JFrame MainWindow			=	new WaterMark();
	}
	
	
	/*
	 * SourceButtonAction.
	 * Gets the Source file path and updates the UI.
	 */
	class SourceButtonAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			int retval			=	SourceFileChooser.showOpenDialog( WaterMark.this );
			if (retval == JFileChooser.APPROVE_OPTION)
			{
				File file		=	SourceFileChooser.getSelectedFile();

				SourceFileBox.setText( file.getPath() );
			}
		}
	}
	
	
	/*
	 * WaterMarkButtonAction.
	 * Gets the Watermark file path and updates the UI.
	 */
	class WaterMarkButtonAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			int retval			=	WatermarkFileChooser.showOpenDialog( WaterMark.this );
			if (retval == JFileChooser.APPROVE_OPTION)
			{
				File file		=	WatermarkFileChooser.getSelectedFile();

				WatermarkFileBox.setText( file.getPath() );
			}
		}
	}	
	
		
	/*
	 * FormatChooserAction.
	 * Assigns the chosen image format.
	 */
	class FormatChooserAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
		        JComboBox tmpSelection		=	(JComboBox)ae.getSource();
        		ChosenFormat			=	(String)tmpSelection.getSelectedItem();
		}
	}
	
	
	/*
	 * PositionChooserAction.
	 * Allows the user to select the position of the watermark.
	 */
	class PositionChooserAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			JComboBox tmpSelection		=	(JComboBox)ae.getSource();
			String _ChosenPosition		=	(String)tmpSelection.getSelectedItem();
			
			if ( _ChosenPosition == "centre" )
			{
				ChosenPosition		=	1;
			}
			else
			{
				ChosenPosition		=	2;
			}
		}
	}
	
	/*
	 * ButtonCreateAction.
	 *
	 */
	class ButtonCreateAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{	
			try
			{
				/*
				 * Call MakeWatermark.
				 */
				MakeWatermark(
					SourceFileBox.getText(),
					WatermarkFileBox.getText(),
					"Output",
					ChosenFormat,
					ChosenPosition
				);
				
				/*
				 * Display Image.
				 */
				File OutFile		=	new File( "Output." + ChosenFormat );
				BufferedImage OutImage	=	ImageIO.read(OutFile);
				
				JFrame ShowOutput	=	new JFrame("WaterMark Output");
				ShowOutput.getContentPane().add( new JLabel( new ImageIcon( OutImage ) ) );
				ShowOutput.pack();
				ShowOutput.setLocationRelativeTo( null );
				ShowOutput.setVisible( true );		
				
			}
			catch( IOException e)
			{
				// Do nothing
			}
		}
	}
	
	
	/*
	 * MakeWatermark( Source, Watermark, Output, Format, Position).
	 * Creates a watermarked image.
	 */
	public static void MakeWatermark( String InputA, String InputB, String OutFile, String OutFormat, int WatermarkPosition ) throws IOException
	{
		try
		{
			/*
			 * Load Source Image.
			 */
			File SourceFile			=	new File( InputA );
			BufferedImage SourceImage 	=	ImageIO.read(SourceFile);
			
			
			/*
			 * Load Watermark Image.
			 */
			File WatermarkFile		=	new File( InputB );
			BufferedImage WatermarkImage	=	ImageIO.read( WatermarkFile );
			
			
			/*
			 * Create canvas.
			 */
			Graphics2D Canvas		=	SourceImage.createGraphics();
			Canvas.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, 0.4f ) );
			
			
			/*
			 * Determine Watermark position.
			 *
			 * 1: Center.
			 * 2: Bottom Right.
			 */
			int _WatermarkPosition;
			int WaterMarkX;
			int WaterMarkY;

			switch ( WatermarkPosition )
			{					
				case 1:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() ) / 2;
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() ) / 2;
				break;
							
				case 2:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() );
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() );
				break;
				
				default:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() ) / 2;
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() ) / 2;
				break;
			}
			
					
			/*
			 * Draw watermark on canvas.
			 */
			Canvas.drawImage( WatermarkImage, WaterMarkX, WaterMarkY, null );
			
			
			/*
			 * Clean up.
			 */
			Canvas.dispose();
 		 	
			
			/*
			 * Write out the file.
			 */
			ImageIO.write(
				SourceImage,
				OutFormat,
				new File( OutFile + '.' + OutFormat )
			);
			
		}
		catch ( IOException e )
		{
			System.out.println("Error: " + e );
			// Do nothing
		}
	}
}
